DROP TABLE DOCQUEUE;
DROP TABLE LANGUAGES CASCADE;
DROP TABLE HEADERS CASCADE;
DROP TABLE COMPANY CASCADE;
DROP TABLE NOTES CASCADE;
DROP TABLE PRINTERS CASCADE;
DROP TABLE DRIVERS CASCADE;
DROP TABLE TABLES;
DROP TABLE WAITERS CASCADE;
DROP TABLE CUSTOMERS CASCADE;
DROP TABLE SUPPLIERS CASCADE;
DROP TABLE PROFILES CASCADE;
DROP TABLE RECIPE_TYPES CASCADE;
DROP TABLE COOKBOOK CASCADE;
DROP TABLE RECIPE_INGREDIENTS;
DROP TABLE RECIPE_VARIANTS;
DROP TABLE PLACEMENTS CASCADE;
DROP TABLE ORDERS CASCADE;
DROP TABLE ORDER_VARIANTS;
DROP TABLE FRIDGE;
DROP TABLE WAREHOUSE;
DROP TABLE PLACEMENTS_ARCHIVE CASCADE;
DROP TABLE ORDERS_ARCHIVE CASCADE;
DROP TABLE ORDER_VARIANTS_ARCHIVE;
DROP TABLE DATA_EXPORT_QUERY;

DROP SEQUENCE receipt_serial_number_seq;
DROP SEQUENCE invoice_serial_number_seq;
DROP SEQUENCE delivery_serial_number_seq;
DROP SEQUENCE order_serial_number_seq;
DROP SEQUENCE refresh_tables_view_seq;
DROP SEQUENCE clients_number_seq;


DROP LANGUAGE 'plpgsql' CASCADE;


CREATE TABLE DOCQUEUE (

	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('docqueue_id_seq') + 1,
	MachineIpAddress TEXT,
	NumTable 	TEXT,
	PrinterName 	TEXT, 
	ConvertCommand TEXT,
	IconvCommand  TEXT,	
	PrintCommand   TEXT,	
	IpAddress TEXT,
	Port TEXT,
	Filename TEXT ,
	CopyNumber 	INT,
	JobStart	     	Time
				DEFAULT CURRENT_TIME,
	Printed TEXT 	CHECK( Printed IN  ('False', 'True'))
				DEFAULT 'False'
	
	
);


CREATE TABLE LANGUAGES (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('languages_id_seq') + 1,
	Language	TEXT,
	File			TEXT,
	HelpUrl		TEXT,
	AdvertisingUrl	TEXT

);



CREATE TABLE HEADERS (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('headers_id_seq') + 1,
	Text  		TEXT
);


CREATE TABLE COMPANY (

	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('company_id_seq') + 1,
	Type			TEXT,
	Header  		TEXT,
	BillingAddress	TEXT,
	ShippingAddress TEXT

);


CREATE TABLE NOTES (
	ID 			SERIAL PRIMARY KEY,
	Position   	INT
				DEFAULT nextval('notes_id_seq') + 1,
	Note  		TEXT
);


CREATE TABLE DRIVERS (
	ID 			SERIAL PRIMARY KEY,
	Name  		TEXT,
	DriverName	TEXT
				DEFAULT('EpsonTM-T88'),
	SerialDriverName	TEXT
				DEFAULT('EpsonTM-T88Serial'),
	LanDriverName	TEXT
				DEFAULT('EpsonTM-T88Lan')

);

CREATE TABLE PRINTERS (
	ID 			SERIAL PRIMARY KEY,
	Name  		TEXT,
	Type 		TEXT CHECK( Type IN  ('Order', 'PreFiscal', 'Fiscal', 'Receipt', 'Invoice', 'Delivery', 'None')),
	IPAddress	TEXT,
	Port			TEXT,
	CharSize		INT,
	idDriver		INT
				NOT NULL 
				REFERENCES DRIVERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
	Show		TEXT CHECK( Show IN  ('False', 'True'))
				DEFAULT 'True'
				
	
);


CREATE TABLE TABLES (
	ID 			SERIAL PRIMARY KEY,
	Number  		TEXT UNIQUE,
	NumChairs  	INT,       
	Status 		TEXT CHECK( Status IN  ('Occupied', 'Free', 'NotPaid')),	
	Note                 TEXT,
	X			 INT,
	Y			 INT
);



CREATE TABLE PROFILES (
	ID 			SERIAL PRIMARY KEY,
	Position		INT
				DEFAULT nextval('profiles_id_seq') + 1,
	Name		TEXT,
	ButtonPrice	INT,
	TablesView	INT,
	Orders		INT,
	Tables		INT,
	Waiters		INT,
	Profiles		INT,
	Customers	INT,
	Suppliers		INT,
	Tickets		INT,
	Menu		INT,
	Quantities	INT,
	Refrigerator	INT,
	StockHouse	INT,
	Notes		INT,
	Company		INT,
	Archive		INT,
	Printers		INT,
	Backup		INT,
	Preferences	INT,
	ButtonDelTable INT	
	
		
);



CREATE TABLE WAITERS (
	ID 			SERIAL PRIMARY KEY,
	Name               TEXT,
	Address            TEXT,
	Telephone		 TEXT,
	Cellular            TEXT,
	Note                 TEXT,
	Password		TEXT
				DEFAULT '',
	IDProfile		INT
				NOT NULL 
				REFERENCES PROFILES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE

);


CREATE TABLE CUSTOMERS (
	ID 			SERIAL PRIMARY KEY,
	Name 		TEXT,
	BillingAddress 	TEXT,
	DeliveryAddress TEXT,
	Telephone	TEXT,
	Mail			TEXT,
	LicenseNumber TEXT,
        FiscalCode    	TEXT,
	Credit		REAL,
	Note 		TEXT,
	Barcode		TEXT  UNIQUE
				DEFAULT nextval('customers_id_seq') + 1,
	Timestamp 	Timestamp
				DEFAULT CURRENT_TIMESTAMP

);


CREATE TABLE SUPPLIERS (
	ID 			SERIAL PRIMARY KEY,
	Name 		TEXT,
	Address         	TEXT,
	Telephone	TEXT,
	Mail			TEXT,
	LicenseNumber TEXT,
        FiscalCode    	TEXT,
	Credit		REAL,
	BANK		TEXT,
	IBAN			TEXT,
	BIC			TEXT,
	Note 		TEXT
	
);


CREATE TABLE RECIPE_TYPES (
	ID 			SERIAL PRIMARY KEY,
	Position		INT
				DEFAULT nextval('recipe_types_id_seq') + 1,
	Name		TEXT,
	Department 	INT
				DEFAULT 1,
	TaxRate 		INT
				DEFAULT 0


);	


CREATE TABLE COOKBOOK (
	ID 			SERIAL PRIMARY KEY,
	IDType		INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name              TEXT,
	Price              	REAL,
	Visible		TEXT CHECK( Visible IN  ('No', 'Yes')),		
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE,
	ButtonColor	TEXT
				DEFAULT 'lightGray',
	ButtonIcon	TEXT,
	Barcode		TEXT,
	AvailableQuantity  	REAL
					DEFAULT 0,
	ShortageQuantity     REAL
					DEFAULT 0,
	ReorderQuantity 	REAL 
					DEFAULT 0,
	PurchasePrice          REAL,
	Supplier			INT
					REFERENCES SUPPLIERS(ID) 
					ON DELETE RESTRICT
					ON UPDATE CASCADE
	
);


CREATE TABLE RECIPE_INGREDIENTS (
	ID 			SERIAL PRIMARY KEY,
	IDRecipe         INT
				NOT NULL 
				REFERENCES COOKBOOK(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name              	TEXT,
	Quantity    	REAL
	
);

CREATE TABLE RECIPE_VARIANTS (
	ID 			SERIAL PRIMARY KEY,
	IDRecipe         INT
				NOT NULL 
				REFERENCES COOKBOOK(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	IDRecipeType   INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,

	Name              TEXT,
	Price                REAL,
	ButtonColor	TEXT
				DEFAULT 'lightGray',
	ButtonIcon	TEXT
	
	
);


CREATE TABLE PLACEMENTS (
	ID 			SERIAL PRIMARY KEY,
	NumTable	 	TEXT UNIQUE,	
	Customer	INT
				REFERENCES CUSTOMERS(ID) 
					ON UPDATE CASCADE,
	
	CustomerName TEXT
				 DEFAULT '',

	Waiter	 	INT
				REFERENCES WAITERS(ID) 
					ON UPDATE CASCADE,	

	Note	 	TEXT,
	BegTimestamp 	Timestamp
					DEFAULT CURRENT_TIMESTAMP,
	EndTimestamp 	Timestamp,
	Total		REAL,
	Working		TEXT CHECK( Working IN  ('False', 'True'))
				DEFAULT 'False'

);


CREATE TABLE ORDERS (
	ID 			SERIAL PRIMARY KEY,
	IDPlacement   INT
				NOT NULL 
				REFERENCES PLACEMENTS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	Position		INT,
	Recipe      	   TEXT,
	Number    	   REAL,
	Price      	  	   REAL,
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE,
	Printed		TEXT CHECK( Printed IN  ('No', 'Yes')),
	IDRecipeType	INT
				NOT NULL 
				REFERENCES RECIPE_TYPES(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE
	

);


CREATE TABLE ORDER_VARIANTS (
	ID 			SERIAL PRIMARY KEY,
	IDOrder 		INT
				NOT NULL 
				REFERENCES ORDERS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE,
	
	Name      			 TEXT,
	Quantity    		 REAL,
	Price      	  		 REAL

);


CREATE TABLE FRIDGE (

	ID 			SERIAL PRIMARY KEY,
	Ingredient	TEXT UNIQUE,
	Quantity  		REAL

);	


CREATE TABLE WAREHOUSE (

	ID 			SERIAL PRIMARY KEY,
	Ingredient	TEXT UNIQUE,
	Quantity  		REAL
				DEFAULT 0,
	ShortageQuantity     REAL
					DEFAULT 0,
	ReorderQuantity 	REAL
					DEFAULT 0,
	PurchasePrice          REAL,
	Supplier			INT
					REFERENCES SUPPLIERS(ID) 
					ON DELETE CASCADE
					ON UPDATE CASCADE

);	


CREATE TABLE PLACEMENTS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	NumTable	 	TEXT,	
	Customer		TEXT,	
	Waiter		TEXT,
	Note	 	TEXT,
	BegTimestamp 	Timestamp
					DEFAULT CURRENT_TIMESTAMP,
	EndTimestamp 	Timestamp,
	Credit		REAL,
	UseCredit		TEXT CHECK( UseCredit IN  ('No', 'Yes')),
	FiscalDocType  TEXT,
	Total		REAL,
	Discount		REAL,
	TotalDiscounted REAL,
	Cash		REAL,
	CreditCard	REAL,
	Bancomat	REAL,
	Other		REAL,
	Paid			REAL,
	NotPaid		REAL,
        SerialNumber	TEXT,
        Taxable            REAL,
        TaxRate            REAL,
        Tax                   REAL
	
	
);


CREATE TABLE ORDERS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	IDPlacementArchive   	INT
						NOT NULL 
						REFERENCES PLACEMENTS_ARCHIVE(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	IDRecipeType  		 	INT
						NOT NULL 
						REFERENCES RECIPE_TYPES(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	Position		   INT,
	Recipe      	   TEXT,
	Number    	   INT,
	Price      	  	   REAL,
	IDPrinter 		INT
				NOT NULL
				REFERENCES PRINTERS(ID) 
					ON DELETE CASCADE				
					ON UPDATE CASCADE
	

);


CREATE TABLE ORDER_VARIANTS_ARCHIVE (
	ID 			SERIAL PRIMARY KEY,
	IDOrderArchive 		INT
						NOT NULL 
						REFERENCES ORDERS_ARCHIVE(ID) 
							ON DELETE CASCADE
							ON UPDATE CASCADE,
	
	Name      			 TEXT,
	Quantity    		 REAL,
	Price      	  		 REAL

);



CREATE TABLE DATA_EXPORT_QUERY (

	ID 			SERIAL PRIMARY KEY,
	Query		TEXT,
	FieldSeparatorCode  INT,
	OutputFileName TEXT

);



CREATE LANGUAGE 'plpgsql' HANDLER plpgsql_call_handler LANCOMPILER 'PL/pgSQL';


CREATE FUNCTION unsetTableStatus() RETURNS opaque AS '

	DECLARE 

	BEGIN

		update TABLES set Status = ''Free'' where Number = OLD.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER unsetTableStatusTrigger
			AFTER DELETE
			ON PLACEMENTS FOR EACH ROW
			EXECUTE PROCEDURE unsetTableStatus();



CREATE FUNCTION setTableStatus() RETURNS opaque AS '

	DECLARE 

	BEGIN

		update TABLES set Status = ''Occupied'' where Number = NEW.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER setTableStatusTrigger
			AFTER INSERT
			ON PLACEMENTS FOR EACH  ROW
			EXECUTE PROCEDURE setTableStatus();



CREATE FUNCTION updateTableStatus() RETURNS opaque AS '

	DECLARE 

	BEGIN

		update TABLES set Status = ''Free'' where Number = OLD.NumTable;
		update TABLES set Status = ''Occupied'' where Number = NEW.NumTable;

		return NEW;
	END;

' LANGUAGE 'plpgsql';



CREATE TRIGGER updateTableStatusTrigger
			AFTER UPDATE
			ON PLACEMENTS FOR EACH  ROW
			EXECUTE PROCEDURE updateTableStatus();




CREATE FUNCTION toint(text) RETURNS integer AS '

        DECLARE 

              _strvalue ALIAS FOR $1;
			_value integer;

        BEGIN
				IF _strvalue = '''' THEN
					RETURN 0;
				ELSE
					 SELECT INTO _value CAST(_strvalue AS INT);

					RETURN _value;
				END IF;

        END;

' LANGUAGE 'plpgsql';



CREATE SEQUENCE receipt_serial_number_seq MINVALUE 0;
SELECT nextval('receipt_serial_number_seq');

CREATE SEQUENCE invoice_serial_number_seq MINVALUE 0;
SELECT nextval('invoice_serial_number_seq');

CREATE SEQUENCE delivery_serial_number_seq MINVALUE 0;
SELECT nextval('delivery_serial_number_seq');

CREATE SEQUENCE order_serial_number_seq MINVALUE 0;
SELECT nextval('order_serial_number_seq');

CREATE SEQUENCE refresh_tables_view_seq MINVALUE 0;
SELECT nextval('refresh_tables_view_seq');

CREATE SEQUENCE clients_number_seq MINVALUE 0;
SELECT nextval('clients_number_seq');


